(* This is an include file. These routines are inserted into the main form during
   compile and are just an extension of the main file.
   It is a mechanism for removing clutter only.
   Needs radiogroup1 on main form and the variables located in the interface and
   implementation definitions*)

uses dateutils;

var SendSerial, ControlAnt, UpdateTimeFlag,   {these variables should be visible to Doug's main program}
    AntennaSet, CancelAnt                     :boolean;
    UTCdateAndTime                            :TdateTime;
    Checkbyte                                 :integer;
    Mode                                      :byte;

(* start radiogroup operations *********************************)

procedure TForm1.RadioGroup1Click(Sender: TObject);
Var Pathname1:string;
    Bufferout:word;
    
(* first some support procedures to the radiogroup*)

  procedure SaveSettings;      {saves the com port settings to file}
  begin
       GetDir(0,Pathname1);
       ComPort1.StoreSettings(stIniFile, pathname1+'\SerialPort.ini');
  end;

  procedure GetSettings;        {loads the com port settings from file}
  begin
       GetDir(0,Pathname1);
       ComPort1.LoadSettings(stIniFile, pathname1+'\SerialPort.ini')
  end;

  procedure DataOnly;          {allows data only - no beam control}
  Var wordtest,Fred:word;
  begin
       ComDataPacket1.StartString := 'A ';             {packet waits for a 'A '}
       ComDataPacket1.StopString := #13;               {stops at <CR>, packet length set to 24}
       if comport1.Connected=false then comport1.open; {ensure port open}
       if antennaset then CancelAnt:=true              {make sure not stuck in antenna control mode}
          else CancelAnt:=false;
       sendserial:=true;                               {permission to send moon data}
       controlant:=false;                              {no permission to send antenna info}
  end;

  procedure NoSerial;                                  {no data communications}
  begin
       if AntennaSet then Dataonly;                    {first make sure that not stuck in Antenna setting mode}
       if comport1.connected then comport1.close;      {ensure port closed}
       sendserial:=false;                              {no permission to send moon data}
       controlant:=false;                              {no permission to send antenna info}
  end;

  procedure DataAndControl;                            {sends data and controls the antenna}
  begin
       ComDataPacket1.StartString := 'A ';             {packet waits for a 'A '}
       ComDataPacket1.StopString := #13;               {stops at <CR>, packet length set to 24}
       if comport1.Connected=false then comport1.open; {ensure port open}
       sendserial:=true;                               {permission to send moon data}
       controlant:=true;                               {permission to send antenna info}
  end;

  procedure UpdatePort;                                {brings up the setup of comms}
  begin
       comport1.close;                                 {ensure port closed before changingport}
       ComPort1.ShowSetupDialog;                       {bring up setup screen}
       SaveSettings;                                   {save the values to serial.ini}
       RadioGroup1.ItemIndex:=0;                       {jump back to first item - port closed}
       sendserial:=false;                              {no permission for data or antenna}
       controlant:=false;
  end;

  procedure UpdateSUtime;                              {updates the time on the beam control}
  begin
       if comport1.connected = false then              {ensure port open}
       begin
        ComDataPacket1.StartString := 'A ';
        ComDataPacket1.StopString := #13;
        comport1.Open;
       end;
       UpdateTimeFlag:=true;                           {prepare for time update}
       sendserial:=true;                               {allow data to be sent}
       controlant:=false;
       radiogroup1.ItemIndex:=1;                       {jump back to first item - port closed}
  end;

(*now the main procedure statement for radiogroup1*)

begin
  Case radiogroup1.ItemIndex of
     0: NoSerial;
     1: DataOnly;
     2: DataAndControl;
     3: UpdatePort;
     4: UpdateSUtime;
  end{case}
end;

(* end of radiogroup operations *********************************)

(* start of time routines ***************************************)

procedure GetTimeZoneBias;                             {get the UTC time from computer}
var  TZoneInfo: TTimeZoneInformation;
     TimeZoneBias: longint;
     DaylightSavingSet:byte;
begin
     DaylightSavingSet:=gettimezoneinformation(Tzoneinfo);
     TimeZoneBias := TZoneInfo.Bias;
     UTCdateAndTime:=IncMinute(Now,TimeZoneBias);
     if DaylightSavingSet = 2  then
      begin
      UTCdateAndTime:=IncMinute(UTCDateandTime,Tzoneinfo.Daylightbias);
      end;
end;

procedure UpDateTime(var Sec:byte;var azout,elout:word;var mode:byte);
begin                                                  {set up the variables for sending}
     GetTimeZoneBias;
     DecodeDateTime(UTCDateAndTime, UTCyr, UTCmth, UTCday, UTChr, UTCmin, UTCsec, AMilliSecond);
     sec:=UTCsec+192;
     azout:=UTCmin * 256;
     azout:=azout+UTChr;
     elout:=UTCday * 256;
     elout:= elout + UTCmth;
     mode:= UTCyr - 2000;
end;
(* end of date time routines************************************************)

(* this is the send data out routine****************************************)

procedure TForm1.ComPort1CTSChange(Sender: TObject; OnOff: Boolean);
var azout, elout, bufferout:word;
    flagout:byte;
begin
  if Onoff and sendserial then
  begin
    azout:= trunc(100*MoonAZ);        {convert float to integer for transmission}
    elout:= trunc(abs(100*MoonEL));
    if MoonEL <0 then
      begin
        elout:=36000+elout;
      end;
    if controlant then                {next two statements concern external antenna control}
      begin                           {initiate external mode in VK5DJ shack unit}
           controlant:=false;
           BufferOut:=comport1.WriteStr(inttostr(68)+' '+inttostr(azout)+' '+inttostr(elout)+' '+inttostr(68)+' '+#13);
           AntennaSet:=true;
      end;
    if CancelAnt then                 {cancel externalmode in VK5DJ shack unit}
      begin
           Cancelant:=false;
           BufferOut:=comport1.WriteStr(inttostr(69)+' '+inttostr(azout)+' '+inttostr(elout)+' '+inttostr(69)+' '+#13);
      end;
    mode:=0;

    if Dn then mode:=Mode + 16;                      {store the direction values into mode}
    if Up then mode:=Mode + 8;
    if Lt then mode:=Mode + 4;
    if Rt then mode:=Mode + 2;

   flagout:=67;                                      {set as a 'C' for normal operation}
    if UpdateTimeFlag then                           {the updatetime button has been pressed}
     begin
         UpdateTime(flagout,azout,elout,mode);       {get the utc time into the variables}
         if checkbyte=mode then                      {if the shack unit replied it has the time}
           begin                                     {reset things for normal operation}
              UpdateTimeFlag:=false;                 {the shack unit echoed the year back correctly}
              mode:=0;                               {ensure beam doesn't move}
              flagout:=67;                           {make sure we're back in normal mode (67='C' is signal to shack unit}
           end;
     end;                                            {now send it out}
  BufferOut:=comport1.WriteStr(inttostr(flagout)+' '+inttostr(azout)+' '+inttostr(elout)+' '+inttostr(mode)+' '+#13);
  mode:=0;                                           {reset mode to free up mode for other purposes}
  end;
end;

(* end of send data out routines *******************************)

(* this is the receive data routine******************************)

procedure TForm1.ComDataPacket1Packet(Sender: TObject; const Str: String);
type str20=string[20];
var tempstring, AZnum, ELnum, Checkstr:str20;
begin
    checkstr:=str;                                     {assign packet to checkstr}
    tempstring:=copy(str,1,length(str));
    AZnum:=copy(tempstring,1,pos(' ',tempstring)-1);   {get antenna AZ}
    delete(tempstring,1,pos(' ',tempstring));
    ELnum:=copy(tempstring,1,pos(' ',tempstring)-1);   {get antenna EL}
    delete(tempstring,1,pos(' ',tempstring));
    CheckStr:=copy(tempstring,1,1);                    {get checkbyte value from Shack unit}
    AntennaAZ:=StrToFloat(AZnum);
    AntennaEl:=StrTofloat(ELnum);
    Checkbyte:=StrToInt(CheckStr);
    label6.caption:=format('%*.*f',[5,1,AntennaAZ]);   {put the values on screen}
    label7.caption:=format('%*.*f',[5,1,AntennaEL]);
end;

(* this is the end of receive data routine******************************)


